@@ -17,16 +17,15 @@ import threading
 import socket
 import pickle
 import numpy as np
-from mpi4py import MPI
 import subprocess
 from time import sleep, time
 from functools import partial
 from typing import Iterable, types
 import matplotlib.pyplot as plt
 import traceback
-from termcolor import colored
 from copy import copy
 import itertools
+import warnings
 
 import numpipe
 from numpipe import slurm, display, notify, mpl_tools, config
@@ -41,6 +40,8 @@ class scheduler:
     """Deferred function evaluation and access to cached function output"""
 
     def __init__(self, dirpath=None):
+        warnings.simplefilter("default")
+
         self.blocks = dict()
         self.instances = dict()
         self.instance_counts = dict()
@@ -61,10 +62,6 @@ class scheduler:
 
         self.filename = os.path.splitext(os.path.basename(sys.argv[0]))[0]
 
-        self.logfile = pathlib.Path(self.dirpath) / f'{self.filename}.log'
-        logging.basicConfig(filename=self.logfile, filemode='w', level=logging.INFO,
-                            format='%(levelname)s: %(message)s')
-        logging.captureWarnings(True)
 
         if USE_SERVER:
             address = ('localhost', 6000)
@@ -73,7 +70,6 @@ class scheduler:
             send_msg(self.pipe, pickle.dumps(['new', 'ID']))
 
         self.complete = False
-        self.mpi_rank = MPI.COMM_WORLD.Get_rank()
         self.notifications = []
 
     #TODO implement load all, jdefer
@@ -121,7 +117,12 @@ class scheduler:
         return self.blocks[label].target.load()
 
     def execute(self):
+        warnings.warn('use scheduler.run() instead of scheduler.execute()', DeprecationWarning)
+        self.run()
+
+    def run(self):
         """Run the requested cached functions and at-end functions"""
+        self._init_logging()
         self.args = run_parser()
         numpipe._pbars.mininterval = self.args.mininterval
         numpipe._pbars.character = config.get_config()['progress']['character']
@@ -157,17 +158,16 @@ class scheduler:
                     labels = self.get_labels(name)
                     blocks_to_execute.update({label: self.blocks[label] for label in labels})
 
-            self.resolve_dependencies_down(blocks_to_execute)
+            for name in self.args.exclude:
+                for key in self.get_labels(name):
+                    blocks_to_execute.pop(key, 0)
 
+            self.resolve_dependencies_down(blocks_to_execute)
             self.num_blocks_executed = len(blocks_to_execute)
-            aborting = False
-            if self.mpi_rank == 0:
-                overwriten = self._overwrite([block.target for block in blocks_to_execute.values()])
-                if not overwriten:
-                    aborting = True
-                    display.abort_message()
-            aborting = MPI.COMM_WORLD.bcast(aborting, root=0)
-            if aborting:
+
+            overwriten = self._overwrite([block.target for block in blocks_to_execute.values()])
+            if not overwriten:
+                display.abort_message()
                 return
 
             self.resolve_dependencies_up(blocks_to_execute)
@@ -203,7 +203,7 @@ class scheduler:
                         for name in remaining:
                             block = blocks_to_execute[name]
                             if self.ready_to_run(block):
-                                execute_block_debug(block, name, self.mpi_rank, self.instances,
+                                execute_block_debug(block, name, self.instances,
                                          self.args.cache_time, num_blocks_ran, self.num_blocks_executed)
                                 to_delete.append(name)
                                 block.complete = True
@@ -225,7 +225,7 @@ class scheduler:
                                 block = blocks_to_execute[name]
                                 if self.ready_to_run(block):
                                     results[name] = pool.apply_async(execute_block, 
-                                            (block, name, self.mpi_rank, self.instances, self.args.cache_time, num_blocks_ran, self.num_blocks_executed))
+                                            (block, name, self.instances, self.args.cache_time, num_blocks_ran, self.num_blocks_executed))
                                     to_delete.append(name)
                                     num_blocks_ran += 1
 
@@ -258,7 +258,7 @@ class scheduler:
 
                         self.complete = True
 
-                        if blocks_to_execute and self.mpi_rank == 0:
+                        if blocks_to_execute:
                             self.notifications.append(partial(notify.send_finish_message,
                                                         filename=self.filename, 
                                                         njobs=len(blocks_to_execute),
@@ -277,15 +277,14 @@ class scheduler:
         
 
         ### At-end functions
-        if self.mpi_rank == 0:
-            if self.at_end_functions and not self.args.no_at_end:
-                display.at_end_message()
+        if self.at_end_functions and not self.args.no_at_end:
+            display.at_end_message()
 
-                for func in self.at_end_functions.values():
-                    func()
-            else:
-                if self.args.notify:
-                    self.send_notifications(check_idle=False, idle=True)
+            for func in self.at_end_functions.values():
+                func()
+        else:
+            if self.args.notify:
+                self.send_notifications(check_idle=False, idle=True)
 
     def ready_to_run(self, block):
         if block.dependencies is None:
@@ -530,20 +529,17 @@ class scheduler:
             add_single_animation(anim)
 
     def send_notifications(self, **kwargs):
-        if self.mpi_rank == 0:
-            t = threading.Thread(target=partial(notify.send_notifications, 
-                                           notifications=self.notifications,
-                                           delay=self.args.notify_delay,
-                                           **kwargs))
-            t.start()
+        t = threading.Thread(target=partial(notify.send_notifications, 
+                                       notifications=self.notifications,
+                                       delay=self.args.notify_delay,
+                                       **kwargs))
+        t.start()
 
     def shared(self, class_type):
         """decorator to add a class for shared variables"""
         return class_type
 
     def display_functions(self):
-        if not self.mpi_rank == 0:
-            return
         display.display_message(self.blocks, self.instances, self.at_end_functions)
 
     def _clean(self, filepaths):
@@ -552,9 +548,6 @@ class scheduler:
            Argumnets: 
                filepaths      list of filepaths to hdf5 files
         """
-        if not self.mpi_rank == 0:
-            return
-
         if filepaths:
             if not self.args.force:
                 delete = display.delete_message(filepaths)
@@ -573,9 +566,6 @@ class scheduler:
            Argumnets: 
                targets        list of targets to delete
         """
-        if not self.mpi_rank == 0:
-            return
-
         targets_to_delete = list(filter(lambda t: t.exists(), targets))
         filepaths = [target.filepath for target in targets_to_delete]
 
@@ -659,10 +649,9 @@ class scheduler:
                 labels = self.get_labels(name)
                 targets_to_delete.extend([self.blocks[label].target for label in labels])
 
-        if self.mpi_rank == 0:
-            overwriten = self._overwrite(targets_to_delete)
-            if not overwriten:
-                display.abort_message()
+        overwriten = self._overwrite(targets_to_delete)
+        if not overwriten:
+            display.abort_message()
 
         return
 
@@ -679,3 +668,9 @@ class scheduler:
         confirm = self._clean(filepaths)
         if not confirm:
             display.abort_message()
+
+    def _init_logging(self):
+        self.logfile = pathlib.Path(self.dirpath) / f'{self.filename}.log'
+        logging.basicConfig(filename=self.logfile, filemode='w', level=logging.INFO,
+                            format='%(levelname)s: %(message)s')
+        logging.captureWarnings(True)
